"""
Common classes and functions for working with large language models.
"""

from typing import Any, Optional

from miniagents.messages import Message


class UserMessage(Message):
    """
    A message from a user.
    """

    role: str = "user"


class SystemMessage(Message):
    """
    A message that is marked as a system message (a concept in large language models).
    """

    role: str = "system"


class AssistantMessage(Message):
    """
    A message generated by a large language model.
    """

    role: str = "assistant"
    model: Optional[str] = None


def message_to_llm_dict(message: Message) -> dict[str, Any]:
    """
    Convert a message to a dictionary that can be sent to a large language model.
    """
    try:
        role = message.role
    except AttributeError:
        role = "user"

    return {
        "role": role,
        "content": str(message),
    }
